#include <windows.h>
#include <stdlib.h>
#include "marquee.h"
#include "resource.h"

// is the effect turned on?
BOOL g_bFlash = FALSE;

// timer id, some unique number
#define FLASH_TIMER  2244              

// is the timer on?
int g_bTimer = FALSE;

///////////////////////////////////////////////////////////////////
// timer method methods
///////////////////////////////////////////////////////////////////

void StartTimer(HWND hWnd)
{
    // start a timer                                           
    // 25 milliseconds
    if ((g_bTimer = SetTimer(hWnd, FLASH_TIMER,25,NULL)) != 0) 
        g_bFlash = TRUE;                

}

void StopTimer(HWND hWnd)
{
    // stop the timer
    if (g_bTimer)
        KillTimer(hWnd,FLASH_TIMER);

    g_bFlash = FALSE;
    g_bTimer = FALSE;


    // when all done, repaint to remove the flash rect
    InvalidateRect(hWnd,NULL,TRUE);
}

///////////////////////////////////////////////////////////////////
// PeekMessage() loop methods
void LoopFlash(HWND hWnd)
{
    MSG msg;
    // start time for comparison
    DWORD dwTime = GetTickCount();

    while (g_bFlash)
    {
        // deal with messages
        if (PeekMessage (&msg, NULL, 0, 0,PM_NOREMOVE))
        {
            // if quit, abort this so app can shut down
            if (msg.message == WM_QUIT)
            {
                g_bFlash = FALSE;
                return;
            }   
            PeekMessage (&msg, NULL, 0, 0,PM_REMOVE);
            TranslateMessage (&msg);
            DispatchMessage (&msg);
        }
    
        // if enough time has passed, draw the flash rect
        // 25 milliseconds
        if (dwTime + 25 < GetTickCount())       
        {
            Draw(NULL, hWnd);
            // new base time
            dwTime = GetTickCount();
        }
    }
    // when all done, repaint to remove the flash rect
    InvalidateRect(hWnd,NULL,TRUE);
}

void StartLoop(HWND hWnd)
{
    g_bFlash = TRUE;     

    
    // call our loop
    LoopFlash(hWnd);
}

void StopLoop(void)
{
    // this aborts loop
    g_bFlash = FALSE;

}


///////////////////////////////////////////////////////////////////
// various init and clean up methods
///////////////////////////////////////////////////////////////////

void Init(HWND hWnd)
{
    // load bitmaps
    LoadBMPS();

    // adjust the brush origin
    ShiftBrushOrigin();
}

void CleanUp(HWND hWnd)
{        
    // if still around, kill the timer
    if (g_bTimer)
        KillTimer(hWnd,FLASH_TIMER);
    // clean up bmps
    DestroyBMPS();       
}

LRESULT FAR PASCAL WndProc (HWND hAppWnd, UINT msg, 
                            WPARAM wP, LPARAM lP)
{
    BOOL bRet = TRUE;
    switch (msg)
    {
        case WM_CREATE:
            Init(hAppWnd);
            break;
        case WM_DESTROY:
            CleanUp(hAppWnd);               
            PostQuitMessage(0);
            break;
        case WM_COMMAND:
        {
            switch (wP)
            { 
                case IDM_TIMER_START:
                    StartTimer(hAppWnd);
                    break;
                case IDM_TIMER_STOP:
                    StopTimer(hAppWnd);
                    break;
                case IDM_LOOP_START: 
                    StartLoop(hAppWnd);
                    break;
                case IDM_LOOP_STOP:
                    StopLoop();
                    break;
                case IDM_EXIT:
                    PostMessage(hAppWnd,WM_DESTROY,0,0);
                    break;
                default: 
                    break;
            };
        }   
        case WM_TIMER:
            // if enabled, draw the "flash"
            if (g_bFlash)
                Draw(NULL,hAppWnd);
            break;
        case WM_PAINT:
        {
            PAINTSTRUCT ps;
    
            HDC hDC = BeginPaint(hAppWnd, &ps);
    
            // do your normal painting here

            // if enabled, draw the "flash" (see text)
            if (g_bFlash)
                Draw(hDC, hAppWnd);
            EndPaint(hAppWnd, &ps);
            bRet = FALSE;
            break;
        }
        default: 
            break;
    } 
    if (bRet)
        bRet = (BOOL)DefWindowProc(hAppWnd, msg, wP, lP);
    return bRet;
}

///////////////////////////////////////////////////////////////////
// Parse command line to change rectangle coordinates (see text)
///////////////////////////////////////////////////////////////////
void ParseCmdLine(LPSTR szCmd)
{
    int temp[4];
    int i = 0;
    LPSTR cmdLine;
    RECT rect;

    if ( *szCmd ) 
    {
        cmdLine = szCmd;
        temp[i++] = (int)atoi((char*)cmdLine);
        while( *cmdLine && i < 4) 
        {
            if ( *cmdLine == ' ' ) 
            {
                *cmdLine = '\0';
                cmdLine++;
                while ( *cmdLine == ' ' ) 
                    cmdLine++;
                temp[i++] = (int)atoi((char*)cmdLine);
            }
            cmdLine++;
        }
    }
    if (i == 4)
    {
        rect.left   = temp[0];
        rect.top    = temp[1];
        rect.right  = temp[2];
        rect.bottom = temp[3];
        SetMarqueeRect(rect);
    }
}

///////////////////////////////////////////////////////////////////
int PASCAL WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, 
                    LPSTR lpszCmdLine,int nCmdShow)
{
    HWND        hAppWnd;           
    char        szAppName[] = "Marquee Demo";
    MSG         msg ;
    WNDCLASS    wndclass ;
    BOOL        bRet = TRUE;
              
    ParseCmdLine(lpszCmdLine);
  
    if (bRet)
    {
        if (!hPrevInstance)
        {
            wndclass.style         = CS_VREDRAW | CS_HREDRAW ;
            wndclass.lpfnWndProc   = WndProc ;
            wndclass.cbClsExtra    = 0 ;
            wndclass.cbWndExtra    = 0 ;
            wndclass.hInstance     = hInstance ;
            wndclass.hIcon         = NULL;
            wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
            wndclass.hbrBackground = (HBRUSH)GetStockObject 
                                                (LTGRAY_BRUSH) ;
            wndclass.lpszMenuName  = MAKEINTRESOURCE( IDR_MainMenu);
            wndclass.lpszClassName = szAppName ;
            
            if (!RegisterClass (&wndclass))
                bRet = FALSE;
            if (bRet)
            {
                hAppWnd = CreateWindow (szAppName, szAppName,
                            WS_OVERLAPPEDWINDOW | WS_CLIPSIBLINGS,
                            100,100, 500, 500, NULL,NULL,
                            hInstance, NULL) ;
    
                if (hAppWnd)
                {
                    ShowWindow(hAppWnd, nCmdShow);
                    UpdateWindow (hAppWnd);
                }
                else
                    bRet = FALSE;
            }
        }
        else
            bRet = FALSE;
    
        if (bRet)
        {
            while(GetMessage (&msg, NULL, 0, 0))
            {
                TranslateMessage (&msg) ;
                DispatchMessage (&msg) ;
            }
            bRet = msg.wParam;
        }
    }
    return bRet;
}
