#include <windows.h>

                
// coordinates of rectangle to be "flashed"
int g_iUpperLeftX = 100;
int g_iUpperLeftY = 100;
int g_iLowerRightX = 200;
int g_iLowerRightY = 200;

// handles of bitmaps used for effect
HBITMAP g_hHorz;
HBITMAP g_hVert;

// amount to shift the origin of the brush based on coordinates of 
// rectangle
int g_iLeftShift;
int g_iRightShift;
int g_iTopShift;
int g_iBottomShift;

// counter of "flash" iterations, used to shift the origin of brush 
// for the effect
// (see text)
int g_iLoopCnt = 0;

void SetMarqueeRect(RECT rect)
{
    g_iUpperLeftX = rect.left;
    g_iUpperLeftY = rect.top;
    g_iLowerRightX = rect.right;
    g_iLowerRightY = rect.bottom;

}

///////////////////////////////////////////////////////////////////
// this changes the origin of the brush based on the rectangle 
// coordinates
void ShiftBrushOrigin()
{
    // shift origin based on coordinates (see text)
    g_iLeftShift = g_iUpperLeftX % 4;
    g_iRightShift = g_iLowerRightX % 4;

    g_iTopShift = g_iUpperLeftY % 4;
    g_iBottomShift = g_iLowerRightY % 4;

#ifndef _WIN32           
    // compensate for un-explained 16 bit behaviour (see text)
    g_iTopShift += 2;
    g_iBottomShift += 2;
#endif
}

///////////////////////////////////////////////////////////////////
// create the bitmap resources for use
void LoadBMPS()
{                
    BITMAP horz;
    BITMAP vert; 

    BYTE HByte[] = { 0x00, 0x00, 
                    0x0F, 0x00, 
                    0x0F, 0x00, 
                    0x00, 0x00, 
                    0x00, 0x00, 
                    0x0F, 0x00, 
                    0x0F, 0x00, 
                    0x00, 0x00 
                   };
                   
    BYTE VByte[] =  { 0x66, 0x00, 
                    0x66, 0x00, 
                    0x66, 0x00, 
                    0x66, 0x00, 
                    0x00, 0x00, 
                    0x00, 0x00, 
                    0x00, 0x00, 
                    0x00, 0x00
                   };

    horz.bmType = 0;
    horz.bmWidth = 8;
    horz.bmHeight = 8;
    horz.bmWidthBytes = 2;
    horz.bmPlanes = 1;
    horz.bmBitsPixel = 1;

    horz.bmBits = (LPVOID) HByte;

    vert.bmType = 0;
    vert.bmWidth = 8;
    vert.bmHeight = 8;
    vert.bmWidthBytes = 2;
    vert.bmPlanes = 1;
    vert.bmBitsPixel = 1;


    vert.bmBits = (LPVOID) VByte;

    g_hHorz = CreateBitmapIndirect(&horz ); 
    g_hVert = CreateBitmapIndirect(&vert ); 
}

///////////////////////////////////////////////////////////////////
// clean up the bitmap resources 
void DestroyBMPS()
{

    // delete the bitmaps
    DeleteObject(g_hHorz);
    DeleteObject(g_hVert);
}

///////////////////////////////////////////////////////////////////
// this does the actual drawing of the selection rectangle
void Draw(HDC hDC, HWND hWnd)
{
    HBRUSH hHorz = NULL;
    HBRUSH hVert = NULL;

    int iLineWidth = 0;
    int iLineHeight = 0;
    int iHorzIndent = 0;
    int iVertIndent = 0;

    HBRUSH hOldBrush = NULL;

    // do we have a dc?
    BOOL bAllocatedDC = FALSE;
    if (hDC == NULL)
    {                      
        hDC = GetDC(hWnd);
        bAllocatedDC = TRUE;
    }

    // create the brushes
    hHorz = CreatePatternBrush( g_hHorz );
    hVert = CreatePatternBrush( g_hVert );
        
    // do the drawing (see text)

    iLineWidth = 4; // vertical lines
    iLineHeight = 4; // horizontal lines

    // iLineWidth/2 -> start half a line width from edge
    
    // Then, to end half a line width from edge, 
    // means we need to cut the length of the line by a full 
    // line width
    // (the amount of the indent on both ends) 
    // thus -> indent * 2

    iHorzIndent = iLineWidth/2;
    iVertIndent = iLineHeight/2;

    // Top
    UnrealizeObject(hHorz);
#ifdef _WIN32
    SetBrushOrgEx(hDC,g_iLoopCnt,g_iTopShift,NULL);
#else
    SetBrushOrg(hDC,g_iLoopCnt,g_iTopShift);
#endif
    hOldBrush = (HBRUSH)SelectObject(hDC, hHorz);
        
    PatBlt(hDC, g_iUpperLeftX+iHorzIndent, g_iUpperLeftY,   
            g_iLowerRightX-g_iUpperLeftX-(iHorzIndent*2), 
            iLineHeight, PATCOPY);
        
    // Bottom
    
    UnrealizeObject(hHorz);
#ifdef _WIN32
    SetBrushOrgEx(hDC,8-g_iLoopCnt,g_iBottomShift,NULL);
#else
    SetBrushOrg(hDC,8-g_iLoopCnt,g_iBottomShift);
#endif
    SelectObject(hDC, hHorz);
        
    PatBlt(hDC, g_iUpperLeftX+iHorzIndent, 
            g_iLowerRightY-iLineHeight,  
            g_iLowerRightX-g_iUpperLeftX-(iHorzIndent*2), 
            iLineHeight, PATCOPY);
        
    // Left
    UnrealizeObject(hVert);
#ifdef _WIN32
    SetBrushOrgEx(hDC,g_iLeftShift,8-g_iLoopCnt,NULL);
#else
    SetBrushOrg(hDC,g_iLeftShift,8-g_iLoopCnt);
#endif
    SelectObject(hDC, hVert);
        
    PatBlt(hDC, g_iUpperLeftX,  g_iUpperLeftY+iVertIndent, 
            iLineWidth,  
            (g_iLowerRightY-g_iUpperLeftY)-(iVertIndent*2), 
            PATCOPY);
        
    // Right
    UnrealizeObject(hVert);
#ifdef _WIN32
    SetBrushOrgEx(hDC,g_iRightShift,g_iLoopCnt,NULL);
#else
    SetBrushOrg(hDC,g_iRightShift,g_iLoopCnt);
#endif
    SelectObject(hDC, hVert);
        
    PatBlt(hDC, g_iLowerRightX-iLineWidth,  
            g_iUpperLeftY+iVertIndent, iLineWidth,  
            (g_iLowerRightY-g_iUpperLeftY)-(iVertIndent*2), 
            PATCOPY);

    // clean up 
    SelectObject(hDC, hOldBrush);

    DeleteObject(hVert);    
    DeleteObject(hHorz);    

    if (bAllocatedDC)
        ReleaseDC(hWnd,hDC);

    // inc the loop count
    g_iLoopCnt++;
    
    if (g_iLoopCnt > 7)
        g_iLoopCnt = 0;
}

