Option Explicit

            'one of six windows structures
            'designed to control printers
            '#5 is used under windows 95,
            'although, you can use other structures
Type PRINTER_INFO_5
        pPrinterName As String
        pPortName As String
        Attributes As Long
        DeviceNotSelectedTimeout As Long
        TransmissionRetryTimeout As Long
End Type

            'This function gets information about
            'a specific printer
            'Printer must be opened with
            'the OpenPrinter function
Declare Function GetPrinter Lib "winspool.drv" _
    Alias "GetPrinterA" (ByVal hPrinter As Long, _
    ByVal Level As Long, pPrinter As Any, _
    ByVal cbBuf As Long, pcbNeeded As Long) As Long

            'This function allows you to change
            'information regarding a specific printer
Declare Function SetPrinter Lib "winspool.drv" _
    Alias "SetPrinterA" (ByVal hPrinter As Long, _
    ByVal Level As Long, pPrinter As Any, _
    ByVal Command As Long) As Long

            'This function copies string
            'needed to move data from buffer
            'to the structure
Declare Function LstrCpy Lib "kernel32" Alias "lstrcpyA" _
    (ByVal lpString1 As String, _
    ByVal lpString2 As Any) As Long

            'This function displays an error number for
            'the most recent error
Declare Function GetLastError& Lib "kernel32" ()

            'This function closes the current handle
            'to a printer
Public Declare Function ClosePrinter& Lib _
    "winspool.drv" (ByVal hPrinter As Long)

            'This function obtains a handle to
            'a specified printer.
            'This handle is required in order
            'to call all other function
Public Declare Function OpenPrinter& Lib "winspool.drv" _
    Alias "OpenPrinterA" (ByVal pPrinterName As String, _
    phPrinter As Long, ByVal pDefault As Long)

            'This attribute is being set in the
            'structure to put the printer back online
Public Const PRINTER_ATTRIBUTE_WORK_OFFLINE = &H400

Public Sub EnablePrinter(prnName As String, _
    Optional SetOffline As Boolean = False)
  'This function takes two parameters. One is the name
  'of the printer name, just like it appears in the
  'printer manager Second is an optional parameter.
  'If set to true, will take printer offline.
  
  Dim BufferSize As Long
  Dim pInfo5 As PRINTER_INFO_5
  Dim res As Long
  Dim Buffer() As Long
  Dim hPrinter As Long

    On Error GoTo EnablePrinter_Error
            'Make sure that we have a printer to work with
    If prnName = "" Then Exit Sub


            'Obtain a handle to a printer
    res = OpenPrinter(prnName, hPrinter, 0)
            'If res <> 0 then Error occured, 0 if success
    If res = 0 Then GoTo DidNotSwitch

            'Find out how big the buffer should be
    res = GetPrinter(hPrinter, 5, 0, 0, BufferSize)
    
            'resize the buffer
    ReDim Buffer(BufferSize) As Long
    
            'fills the buffer
            'with data. Will later go to Printer_Info_5
    res = GetPrinter(hPrinter, 5, Buffer(0), BufferSize, _
        BufferSize)
        
        
    If res = 0 Then GoTo DidNotSwitch
    
           'Copy from buffer into the structure
           'use a special function for that task
    pInfo5.pPrinterName = ConvertToVbString(Buffer(0))
    pInfo5.pPortName = ConvertToVbString(Buffer(1))
    pInfo5.Attributes = Buffer(2)
    pInfo5.DeviceNotSelectedTimeout = Buffer(3)
    pInfo5.TransmissionRetryTimeout = Buffer(4)
        
        
            'This will change the printer status
            'Depending on the flag
    If SetOffline = False Then
        pInfo5.Attributes = 0
    Else
        pInfo5.Attributes = PRINTER_ATTRIBUTE_WORK_OFFLINE
    End If
            
            'Change the printer settings
    res = SetPrinter(hPrinter, 5, pInfo5, 0)
    If res = 0 Then GoTo DidNotSwitch

            'Done with this printer
    res = ClosePrinter(hPrinter)
    If res = 0 Then GoTo DidNotSwitch

            'Success
    Exit Sub
    
            'Failed
DidNotSwitch:
    MsgBox "Did not switch printer successfully"

    Exit Sub
            'Failed
EnablePrinter_Error:
    MsgBox Error

End Sub


Private Function ConvertToVbString(lngStr As Long) _
    As String

 On Error Resume Next

 Dim strTemp As String * 512
 Dim i As Long
            
            'Convert String
 i = LstrCpy(strTemp, lngStr)
 
            'Check for zeros
 If (InStr(1, strTemp, Chr(0)) = 0) Then
    ConvertToVbString = ""
 Else       'Return VB string
    ConvertToVbString = Left(strTemp, _
        InStr(1, strTemp, Chr(0)) - 1)
 End If
 
End Function
