// wdjsrvc.c

#include <windows.h>
#include <stdio.h>

HANDLE  hDoneEvent = NULL, hThread = NULL;
DWORD   dwCurrentState;
SERVICE_STATUS_HANDLE  hService;
TCHAR szCurrentUser[MAX_PATH], szDbg[MAX_PATH];;
TCHAR szRegPath[] =
 TEXT("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon");
TCHAR szCaption[] = TEXT("WDJSrvc");

BOOL NotifySCM(DWORD, DWORD, DWORD);
VOID WINAPI ServiceMain(DWORD, LPTSTR *);
VOID WINAPI ServiceHandler(DWORD);
BOOL WINAPI CtrlHandler(DWORD);
DWORD WINAPI WorkerThread(LPVOID);
VOID ProcessRegistryChange(HKEY, HANDLE);
VOID ProcessEventLogChange(HANDLE);

//-----------------------------------------------------------------
void main(int argc, char **argv)
{
   SERVICE_TABLE_ENTRY ServiceTable[] = {
      {TEXT("WDJSrvc"), ServiceMain}, {NULL, NULL}
   };
   
   StartServiceCtrlDispatcher(ServiceTable);
}

//-----------------------------------------------------------------
VOID WINAPI ServiceMain(DWORD dwArgc, LPTSTR *lpszArgv)
{
   DWORD   ThreadId;

   if (!(hService = RegisterServiceCtrlHandler(
         TEXT("WDJSrvc"), ServiceHandler))) {
      return;
   }

   NotifySCM(SERVICE_START_PENDING, 0, 1);

   hDoneEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
   hThread = CreateThread(0,0,WorkerThread,0,0,&ThreadId);

   SetConsoleCtrlHandler(CtrlHandler, TRUE);

   NotifySCM(SERVICE_RUNNING, 0, 0);
   WaitForSingleObject(hDoneEvent, INFINITE);

   CloseHandle(hThread);
   CloseHandle(hDoneEvent);
   return;
}

//--------------------------------------------------------------
VOID WINAPI ServiceHandler(DWORD fdwControl)
{
   switch(fdwControl) {
      case SERVICE_CONTROL_STOP:
         NotifySCM(SERVICE_STOP_PENDING, 0, 1);
         SetEvent(hDoneEvent);
         NotifySCM(SERVICE_STOPPED, 0, 0);
         break;

      case SERVICE_CONTROL_INTERROGATE:
         NotifySCM(dwCurrentState, 0, 0);
         break;

      case SERVICE_CONTROL_SHUTDOWN:
         break;
   }
}

//--------------------------------------------------------------
BOOL NotifySCM(DWORD State, DWORD ExitCode, DWORD Progress)
{
   SERVICE_STATUS ServiceStatus;

   ServiceStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
   ServiceStatus.dwCurrentState = dwCurrentState = State;
   ServiceStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP |
                                      SERVICE_ACCEPT_SHUTDOWN;
   ServiceStatus.dwWin32ExitCode = ExitCode;
   ServiceStatus.dwServiceSpecificExitCode = 0;
   ServiceStatus.dwCheckPoint = Progress;
   ServiceStatus.dwWaitHint = 5000;

   return SetServiceStatus(hService, &ServiceStatus);
}

//-----------------------------------------------------------------
BOOL WINAPI CtrlHandler(DWORD dwCtrlType)
{
   if (dwCtrlType == CTRL_LOGOFF_EVENT) {
      lstrcpy(szDbg, TEXT("CTRL_LOGOFF_EVENT"));
   } else {
      lstrcpy(szDbg, TEXT("Some other console control"));
   }
   
   MessageBox(NULL,szDbg,szCaption,MB_OK|MB_SERVICE_NOTIFICATION);
   return FALSE; // pass to next handler
}

//-----------------------------------------------------------------
DWORD WINAPI WorkerThread(LPDWORD ThreadParam)
{
   HKEY hKey = NULL;   
   HANDLE hEvents[2];
   HANDLE hEventLog;
   EVENTLOGRECORD *pLog;  
   BYTE Buffer[1024]; 
   DWORD dwBytesRead, dwBytesNext, Records, status, 
         dwBytes = MAX_PATH; 
 
   // register for notification of WinLogon registry key changes

   hEvents[0] = CreateEvent(NULL, FALSE, FALSE, NULL);
      
   RegOpenKeyEx(HKEY_LOCAL_MACHINE,szRegPath,0,KEY_READ,&hKey);

   RegQueryValueEx(hKey, TEXT("DefaultUserName"), NULL, NULL,
                   szCurrentUser, &dwBytes);
   
   wsprintf(szDbg, TEXT("Currently logged on: %s"), szCurrentUser);
   MessageBox(NULL,szDbg,szCaption,MB_OK|MB_SERVICE_NOTIFICATION);

   RegNotifyChangeKeyValue(hKey,FALSE,REG_NOTIFY_CHANGE_LAST_SET,
                           hEvents[0], TRUE);

   // register for notification of new security event log entries

   hEvents[1] = CreateEvent(NULL, FALSE, FALSE, NULL);
   hEventLog = OpenEventLog(NULL, TEXT("Security"));
   
   GetNumberOfEventLogRecords(hEventLog, &Records);   
   pLog = (EVENTLOGRECORD *)&Buffer; 
   ReadEventLog(hEventLog, EVENTLOG_SEEK_READ |
                EVENTLOG_FORWARDS_READ, Records-1, pLog, 1024, 
                &dwBytesRead, &dwBytesNext);

   NotifyChangeEventLog(hEventLog, hEvents[1]);
 
   while (TRUE) {                                   
      status = WaitForMultipleObjects(2,hEvents,FALSE,INFINITE);
   
      if (status == WAIT_OBJECT_0) {
         ProcessRegistryChange(hKey, hEvents[0]);
      } else if (status == WAIT_OBJECT_0 + 1) {
         ProcessEventLogChange(hEventLog);
      }
   }

   return TRUE;
}

//-----------------------------------------------------------------
VOID ProcessRegistryChange(HKEY hKey, HANDLE hEvent)
{
   TCHAR szUserName[MAX_PATH];
   DWORD dwBytes = MAX_PATH;

   RegQueryValueEx(hKey, TEXT("DefaultUserName"), NULL, NULL,
                   szUserName, &dwBytes);

   if (lstrcmpi(szUserName, szCurrentUser) != 0) {
      lstrcpy(szCurrentUser, szUserName);
      wsprintf(szDbg, TEXT("ProcessRegistryChange: %s logon"), 
               szUserName);
   } else {
      wsprintf(szDbg, TEXT("ProcessRegistryChange: Don't care"));
   }

   MessageBox(NULL,szDbg,szCaption,MB_OK|MB_SERVICE_NOTIFICATION);
   
   RegNotifyChangeKeyValue(hKey,FALSE,REG_NOTIFY_CHANGE_LAST_SET,
                           hEvent, TRUE);
}

//-----------------------------------------------------------------
VOID ProcessEventLogChange(HANDLE hEventLog)
{
   EVENTLOGRECORD *pLog;  
   BYTE Buffer[1024]; 
   DWORD dwBytesRead, dwBytesNext; 
 
   pLog = (EVENTLOGRECORD *)&Buffer; 
 
   while (ReadEventLog(hEventLog,
             EVENTLOG_FORWARDS_READ | EVENTLOG_SEQUENTIAL_READ,
             0, pLog, 1024, &dwBytesRead, &dwBytesNext)) {
   
      while (dwBytesRead > 0) { 
         if (pLog->EventID == 528) {
             MessageBox(NULL,TEXT("EventLog: Log On"),
                      szCaption,MB_OK | MB_SERVICE_NOTIFICATION);
         } else if (pLog->EventID == 538) {
             MessageBox(NULL,TEXT("EventLog: Log Off"),
                      szCaption,MB_OK | MB_SERVICE_NOTIFICATION);
         }
         dwBytesRead -= pLog->Length; 
         pLog = (EVENTLOGRECORD *)((LPBYTE)pLog + pLog->Length); 
      } 
      pLog = (EVENTLOGRECORD *)&Buffer; 
   }
}
