/****************************************************************
$Module       : XOLEPP (xolepp.cpp)
$See Also     : xolepp.h for class definition and further 
                documentation
****************************************************************/

/****************************************************************
   Includes
****************************************************************/
#include "stdafx.h"        // Precompiled header
#include "xolepp.h"        // CExtOlePropertyPage class definition

/****************************************************************
   Debug Setup
****************************************************************/
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/****************************************************************
   Message Map
****************************************************************/
BEGIN_MESSAGE_MAP(CExtOlePropertyPage, COlePropertyPage)
   //{{AFX_MSG_MAP(CExtOlePropertyPage)
   //}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
//
// Construction, Destruction, and Initialization 
//
/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////

/****************************************************************
$Function:     CExtOlePropertyPage
$Description:  Creates an instance of this class
$Type:         Constructor
$Returns:      n/a
****************************************************************/
CExtOlePropertyPage::CExtOlePropertyPage
(
   UINT idDlg, 
   UINT idCaption
) 
   :COlePropertyPage(idDlg, idCaption)
{
   //{{AFX_DATA_INIT(CExtOlePropertyPage)
   //}}AFX_DATA_INIT
}

/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
//
// Data Exchange
//
/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////

/****************************************************************
$Function:     DDP_CBData
$Description:  Exchanges data between an objects property value 
               and the corresponding combo box entry. This loads 
               the combobox with the value that the control 
               supports as reported by the IPerPropertyBrowsing 
               interface.  
               
               This exchanges data as opposed to 0 based indicies 
               like DDP_CBIndex.  By data we mean that the 
               property might have a value of 2 and that might 
               correspond to the 5th combobox entry. There is no 
               longer a restriction on the control to support 0 
               based sequential property values.  This takes 
               care of mapping the value fo the appropriate 
               combobox entry

$Type:         protected, DDP_ transfer function
$Returns:      void

$Usage:        A call to this should be followed with a call to 
               DDX_CBData which will exchange the combobox data 
               with a local member variable.  That same variable 
               should be used as the nData parameter.  This is 
               because the save functionality of this method just 
               saves the address of the variable that is being
               exchanged and does a gang load transfer when 
               DDP_PostProcessing is called.

$Remarks:      - The ActiveX control must implement the 
                 IPerPropertyBrowsing interface.  For MFC controls, 
                 that means implementing the 
                 OnGetPredefinedStrings and OnGetPredefinedValue 
                 virtual functions.  
               - Currently, we only support MFC ActiveX Control 
                 implementations - no ATL.  
*****************************************************************/
void CExtOlePropertyPage::DDP_CBData
( 
   CDataExchange* pDX,           // (I) Data exchange object
   int            id,            // (I) Control id to exchange with
   int&           nData,         // (I/O) Value that is exchanged
   LPCTSTR        pszPropName    // (I) Name of the property to 
                                 // exchange with
)
{
   // Loading the combobox from the property value
   if (!pDX->m_bSaveAndValidate)
   {
      // Get a combobox window from the control id.
      HWND hWndCtrl = pDX->PrepareCtrl(id);
      CComboBox* pwnd = (CComboBox*)CWnd::FromHandle(hWndCtrl);
      if (pwnd != NULL)
      {
         // Clear out the combobox
         pwnd->ResetContent();

         // Get the property page's IDispatch array
         ULONG ulObjects;
         LPDISPATCH* lpObjectArray = GetObjectArray( &ulObjects );
         ASSERT( lpObjectArray != NULL );

         // I'm assuming there is but one control.
         LPDISPATCH lpdispControl = lpObjectArray[0];

         // Get a pointer to the control object.  Doing this 
         // enforces that the control was implemented with MFC.  
         // In order to be more generic, we will need to talk to 
         // the IPerPropertyBrowsing interface directly.
         COleControl* pControl = 
            (COleControl*)CCmdTarget::FromIDispatch(lpdispControl);

         // Get an OLE String for the property name
         int _convert; _convert = 0;
         LPCOLESTR lpOleStr = T2COLE(pszPropName);

         // Get the property id for the specified property name
         DISPID dwDispID;
         HRESULT result = lpdispControl->GetIDsOfNames(
            IID_NULL, (LPOLESTR*)&lpOleStr, 1, 0, &dwDispID);

         if (result == S_OK)
         {
            // Get an array of strings and their associated cookies
            // for each possible value for the specified property
            CStringArray arStr;
            CDWordArray arCookie;
            pControl->OnGetPredefinedStrings(
               dwDispID, &arStr, &arCookie);
            int numItems = arStr.GetSize();

            // Loop through each possible value.  For each one, we 
            // will create a combobox entry that has property 
            // value and it textual  description.
            for (int i=0; i<numItems; i++)
            {
               // Get the data associated with the cookie for this 
               // property value.  If no value is supplied, we 
               // will assume that the values are using 0 based 
               // indexing (the first item has a value of 0, 
               // then 1, etc...)
               long nData = i;
               VARIANT var;
               if (pControl->OnGetPredefinedValue(
                  dwDispID, arCookie[i], &var))
               {
                  if (var.vt == VT_I4)
                     nData = var.lVal;
               }

               // Format a combobox string
               CString strAdd;
               strAdd.Format( "%d - %s", nData, (LPCSTR)arStr[i] );

               // Add item to the combobox.  The property value is 
               // the  item data.
               int idx = pwnd->AddString(strAdd);
               ASSERT(idx != CB_ERR);
               VERIFY(pwnd->SetItemData(idx, nData) != CB_ERR);
            }

         }  // end 'if (result == S_OK)' 

      }  // end 'if (pwnd != NULL)'
   }

   // Transfer the value like it was a numeric value obtained from 
   // and  edit control.  For a load, nData will contain the 
   // property value. For a save, nData will later be filled with 
   // the property value as obtained from DDX_CB_DATA.
   DDP_Text(pDX, id, nData, pszPropName);
}

/****************************************************************
$Function:     DDX_CBData
$Description:  Transfers data between an integer and the item 
               data of the selected combobox entry.  This is 
               different from DDB_CBIndex which simply transfers 
               the index of the selected item. 
$Type:         protected, DDX_ transfer function
$Returns:      void

$Results:      On a load, the combobox will select the item that 
               has the item data with a value of 'nData'.  
               On a save, 'nData' will return the item data of the 
               currently selected combobox item.
*****************************************************************/
void CExtOlePropertyPage::DDX_CBData
( 
   CDataExchange* pDX,     // (I) Data exchange object
   int            nIDC,    // (I) id of the control to transfer 
                           // with
   int&           nData    // (I/O) data to transfer.
)
{
   // Get a combobox window from the control id.
   HWND hWndCtrl = pDX->PrepareCtrl(nIDC);
   CComboBox* pwnd = (CComboBox*)CWnd::FromHandle(hWndCtrl);
   if (pwnd != NULL)
   {
      // Load
      if (!pDX->m_bSaveAndValidate)
      {
         // Loop through the combobox until we find the item with 
         // the corresponding item data
         int nCount = pwnd->GetCount();
         int idx = -1;
         for (int i=0; i<nCount && idx <0; i++)
         {
            if (pwnd->GetItemData(i) == nData)
               idx = i;
         }

         // Transfer the combobox index to the control for normal 
         // selection
         DDX_CBIndex(pDX, nIDC, idx);
      }

      // Save
      else
      {
         // Get the index of the selected item and it's item data.
         int idx;
         DDX_CBIndex(pDX, nIDC, idx);
         nData = pwnd->GetItemData(idx);
      }
   }
}
